@@ -14,10 +14,7 @@ client = httpx.AsyncClient(timeout=TIMEOUT)
 
 def check_for_errors(j: dict) -> None:
     if errors := j.get("errors"):
-        error_strs: T.List[str] = []
-        for error in errors:
-            error_strs.append(error["message"])
-        raise GQLException(error_strs)
+        raise GQLException(errors)
 
 
 def finish(
