@@ -145,6 +145,10 @@ class Node(BaseModel):
         resolver: T.ClassVar[T.Type[ResolverType]]
         payload_node_name: str
 
+        """ TODO fill the add and patch models in future for updating and adding"""
+        add_model: T.Type[BaseModel]
+        patch_model: T.Type[BaseModel]
+
     # TODO make equality and hashing
 
     @property
@@ -255,24 +259,58 @@ class Node(BaseModel):
 
     """CRUDS"""
 
+    @staticmethod
+    def make_set_and_remove_lists(
+        original_list: T.Optional[set], new_list: T.Optional[set]
+    ) -> T.Tuple[set, set]:
+        # add all values from original list not in new list
+        if original_list is None:
+            original_list = set()
+        if new_list is None:
+            new_list = set()
+        if type(original_list) is list:
+            original_list = set(original_list)
+        if type(new_list) is list:
+            new_list = set(new_list)
+        set_list = new_list - original_list
+        remove_list = original_list - new_list
+        return set_list, remove_list
+
     def base_set_remove_d(
         self, to_set: REL_D_INPUT = None, to_remove: REL_D_INPUT = None
     ) -> T.Tuple[GQL_D, GQL_D]:
-        set_d: GQL_D = {}
-        remove_d: GQL_D = {}
+        set_patch = self.Dgraph.patch_model()
+        remove_patch = self.Dgraph.patch_model()
 
-        curr_dict = self.dict()
-        for field_name, val in self._original_dict.items():
-            if field_name not in curr_dict:
-                # it was removed
-                remove_d[field_name] = val
-            elif val != curr_dict[field_name]:
-                # it was changed
-                set_d[field_name] = curr_dict[field_name]
-        # now set all the new ones
-        new_field_names = set(curr_dict.keys()) - set(self._original_dict.keys())
-        for new_field_name in new_field_names:
-            set_d[new_field_name] = curr_dict[new_field_name]
+        current_d = self.dict()
+        original_d = self._original_dict
+
+        unset_key = "_UNSET_12398120"
+
+        for field_name in self.Dgraph.patch_model.__fields__.keys():
+            current_val = current_d.get(field_name, unset_key)
+            original_val = original_d.get(field_name, unset_key)
+            if current_val != original_val:
+                # if list, if values have been removed, add to remove... if values have been added, keep
+                if {type(current_val), type(original_val)} & {list, set}:
+                    set_list, remove_list = self.make_set_and_remove_lists(
+                        original_list=original_val, new_list=current_val
+                    )
+                    setattr(set_patch, field_name, set_list)
+                    setattr(remove_patch, field_name, remove_list)
+                else:
+                    if current_val != unset_key:
+                        # if current val is not unset -> always set
+                        if current_val is not None:
+                            setattr(set_patch, field_name, current_val)
+                        else:
+                            setattr(remove_patch, field_name, current_val)
+                    else:
+                        # current val is UNSET but original val is not
+                        setattr(remove_patch, field_name, original_val)
+
+        set_d: GQL_D = set_patch.dict(exclude_unset=True)
+        remove_d: GQL_D = remove_patch.dict(exclude_unset=True)
 
         # now add relationship fields
         rel_set_d = self.make_gql_d_from_rel_d_input(rel_d_input=to_set or {})
@@ -295,20 +333,21 @@ class Node(BaseModel):
     async def _add(
         cls: T.Type[NodeModel],
         *,
-        input: BaseModel,
+        input: Dgraph.add_model,
         given_resolver: T.Optional[ResolverType] = None,
         upsert: bool = False,
         relationships_input: REL_D_INPUT = None,
     ) -> NodeModel:
         resolver = given_resolver or cls.Dgraph.resolver()
         query_str = resolver.make_add_mutation_str()
+        if not isinstance(input, cls.Dgraph.add_model):
+            input = cls.Dgraph.add_model.parse_obj(input)
+
         input_d = jsonable_encoder(input)
 
         rel_d = cls.make_gql_d_from_rel_d_input(rel_d_input=relationships_input or {})
         input_d.update(rel_d)
-        variables = {"input": input_d, "upsert": upsert}
-        debug(query_str)
-        debug(variables)
+        variables = {"input": jsonable_encoder(input_d), "upsert": upsert}
         j = await gql(query_str=query_str, variables=variables)
         node_d = j["data"][f"add{cls.Dgraph.typename}"][cls.Dgraph.payload_node_name][0]
         return resolver.parse_obj_nested(node_d)
@@ -360,7 +399,11 @@ class Node(BaseModel):
             print("NOTHING TO UPDATE!")
             return False
 
-        variables = {"set": set_d, "remove": remove_d, "filter": {"id": self.id}}
+        variables = {
+            "set": jsonable_encoder(set_d),
+            "remove": jsonable_encoder(remove_d),
+            "filter": {"id": self.id},
+        }
 
         if print_update_d:
             debug(variables)
